#this file is part of manqod
#manqod is distributed under the CDDL licence
#the author of manqod is Dobai-Pataky Balint(dpblnt@gmail.com)

class DrbDb
	include Eprint
	include DRbUndumped
	include SQL
	include Help
	include Cron
	include GtkAttributes
	include EventCache
	include Messaging
	
	INIT=0
	LOADING=1
	SERVING=2

	def initialize(connection,main_server_uri)
		@state=INIT
		@connection=connection
		@main_server_uri=main_server_uri
		@moditems=Hash.new #[moditem_id][id][fields]
		@server=nil
#		@relations=Hash.new
#		@server=DRb::DRbServer.new(@connection['uri'],self)
	end
	attr_reader :uri, :moditems, :admin, :client, :cache, :connection, :images, :state, :relation_builder
	attr_accessor :server, :main_server

	def init
		einfo("Memcached #{Memcached::VERSION}")
	  @cache = Memcached.new([@connection['cache_host']],{:prefix_key=>key_name, :default_ttl => 0, :timeout => 30})
		DRb.start_service(@connection["uri"],self)
		@server=DRb.current_server()
		@main_server=DRb::DRbObject.new_with_uri(@main_server_uri)
		@uri=server.uri()
		unless init_sql(@connection['sql_host'],@connection['sql_user'],@connection['sql_password'],@connection['sql_db'])
			eerror("connection to sql server failed: #{@connection['sql_user']}@#{@connection['sql_host']}/#{@connection['sql_db']}")
			return nil
		end
		update_manqod_db
		begin
			begin
				@admin=DRb::DRbObject.new(nil,@connection['admin_uri'])
				we_have_admin=@admin.alive?
			rescue
				ewarn("no admin?")
				sleep 1
			end
		end until we_have_admin
		begin
			@state=LOADING
			load_all_attributes

			admin.rows("select moditems.id,modules.modname,moditems.display from moditems left join modules on modules.id = moditems.modid").each{|moditem|
				begin
					case moditem["modname"]
						when "listing"
							@moditems[moditem["id"].to_i]=DrbListModel.new(self,moditem["id"].to_i).create_skeleton unless @moditems.has_key?(moditem["id"])
							@moditems[moditem["id"].to_i].update(self)
						when "form"
							@moditems[moditem["id"].to_i]=DrbForm.new(self,moditem["id"].to_i).create_skeleton unless @moditems.has_key?(moditem["id"])
						when "relation_builder"
							@relation_builder=@moditems[moditem["id"].to_i]=DrbRelationBuilder.new(self,moditem["id"].to_i) unless @moditems.has_key?(moditem["id"])
						else
							ewarn("not caching #{moditem['display']}[#{moditem['modname']}]")
					end
				rescue =>err
					ewarn("error loading moditem: #{moditem.inspect}")
					eexception(err,:moditem => moditem)
				end
			}
			@images=DrbImages.new(self).load_all
			@users=Users.new(self).load_all
			load_all_events
			load_all_help
			start_cron
			@state=SERVING
		rescue => err
			eexception(err)
			retry
		end

		DRb.thread.join
		einfo("EXITED")
		exit
	end
	
	def setup_client
		if @connection['client_uri'].length>0
			begin
				@client=DRb::DRbObject.new(nil,@connection['client_uri'])
				einfo("client is alive?(#{@client}): #{@client.alive?}")
				
=begin
				@moditems.each_value{|mi| 
					ecode(mi.class.name)
					mi.update if mi.class == DrbRelationBuilder
				}
=end
			rescue
				@client=nil
				einfo("no client.")
			end
			begin
				@relation_builder.update if @relation_builder && @client
			rescue => err
				eexception(err)
			end
		end
	end
	def name
		@connection['name']
	end
	def key_name
		@connection['key_name']
	end


	def moditem(mod_id)
		@moditems[mod_id.to_i]
	end
	
	def reload_moditem(moditem_id)
		@state=LOADING
		if moditem(moditem_id).nil?
			einfo("creating moditem ##{moditem_id}")
			#moditem added
			admin.rows("select moditems.id,modules.modname,moditems.display
			from moditems 
			left join modules on modules.id = moditems.modid
			where moditems.id='#{moditem_id}'").each{|moditem|
			begin
				case moditem["modname"]
					when "listing"
						@moditems[moditem["id"].to_i]=DrbListModel.new(self,moditem["id"].to_i).create_skeleton unless @moditems.has_key?(moditem["id"])
						@moditems[moditem["id"].to_i].update(self)
					when "form"
						@moditems[moditem["id"].to_i]=DrbForm.new(self,moditem["id"].to_i).create_skeleton unless @moditems.has_key?(moditem["id"])
					else
						eerror("not caching #{moditem['display']}[#{moditem['modname']}]")
				end
			rescue =>err
				ewarn("error loading moditem: #{moditem.inspect}")
				eexception(err,:moditem => moditem)
			end
			}
			
		else
			einfo("reloading moditem ##{moditem_id}")
			case moditem(moditem_id).mod_type
				when "list"
					new_model=DrbListModel.new(self,moditem_id).create_skeleton
					edebug("#{new_model} created")
					new_model.update(self)
					unless @moditems[moditem_id.to_i].nil?
						@moditems[moditem_id.to_i].clients.each_pair{|client_id,client| new_model.subscribe(client_id,client)}
						edebug("#{@moditems[moditem_id.to_i]} abandoned")
					end
					@moditems[moditem_id.to_i]=new_model
					@moditems[moditem_id.to_i].notify_clients(nil,"structure")
				when "form"
					moditem(moditem_id).create_skeleton
			end
		end
		@state=SERVING
	end
	
	def reload_users
		@users.load_all
	end
	
	def reload_client_images
		dead=Array.new
		main_server.connected_clients.each_pair{|client_id,cdb|
			if cdb[:db] == client.name
				begin
					einfo("sending reload images to #{client_id}")
					sdb[:object].rpc("DrbImages.instance.load_images")
				rescue => err
					ewarn("cannot send reload images to #{client_id}")
					dead.push(client_id)
				end
			end
		}
		dead.each{|client_id|
			ewarn("killing dead client: #{client_id}")
			remove_client(client_id)
		}
	end
	
	def reload_client_attributes
		client.load_all_attributes
		dead=Array.new
		main_server.connected_clients.each_pair{|client_id,cdb|
			if cdb[:db] == client.name
				begin
					einfo("sending reload attribute to #{client_id}")
					cdb[:object].rpc("GtkAttributeStorage.instance.load_all")
				rescue => err
					ewarn("cannot send reload attributes to #{client_id}")
					dead.push(client_id)
				end
			end
		}
		dead.each{|client_id|
			ewarn("killing dead client: #{client_id}")
			remove_client(client_id)
		}
	end

	def reload_client_events(goid)
		einfo("changed client event: #{goid}")
		client.events(goid,true)
		dead=Array.new
		main_server.connected_clients.each_pair{|client_id,cdb|
			if cdb[:db] == client.name
				begin
					einfo("sending reload event to #{client_id}")
					cdb[:object].rpc("EventCache.instance.reload_events(\"#{goid}\")")
				rescue => err
					ewarn("cannot send reload events to #{client_id}")
					dead.push(client_id)
				end
			end
		}
		dead.each{|client_id|
			ewarn("killing dead client: #{client_id}")
			remove_client(client_id)
		}
	end

	def auth?(nick,password)
		r=@users.auth?(nick,password)
		edebug("auth?(#{nick}):#{r}")
		r
	end
	def sendmail(nick,subject,body)
		sendmail_to_nick(nick,subject,body)
	end
	def user_in_group?(user_id,group_id)
		@users.user_in_group?(user_id,group_id)
	end
	def mygroups(nick)
		@users.mygroups(nick)
	end
	
	def moditems_with_base(base)
		@moditems.each_value{|drblist|
			yield drblist if drblist && drblist.mod_type == "list" && drblist.base == base
		}
	end
	
	def changed_ids_of_base(base,ids,nick=nil)
		base_moditem=nil
		moditems_with_base(base){|b|
			base_moditem=b
		}
		unless base_moditem.nil?
			base_moditem.rows_changed(ids,nick)
		end
	end

	def lock_id_of_base(base,lid)
		ids=if lid.class == Array then lid else [lid] end
		moditems_with_base(base){|b| ids.each{|iid| b.lock_iter(iid)}}
	end

	def unlock_id_of_base(base,lid)
		ids=if lid.class == Array then lid else [lid] end
		moditems_with_base(base){|b| ids.each{|iid| b.unlock_iter(iid)}}
	end
	
	def each_moditem(modtype = "list")
		@moditems.each_value{|m|
			yield m if m.mod_type == modtype
		}
	end
	
	def remove_dead_clients
		cnt=0
		@moditems.each_value{|m| cnt+=m.remove_dead_clients if m.mod_type == "list"}
		cnt
	end

	def remove_client(client_id)
		@moditems.each_value{|m| m.unsubscribe(client_id) if m.mod_type == "list"}
	end

	def alive?
		true
#		server.alive?
	end

#	def method_missing(sym,*args)
#		p "#{self} missing method: #{sym}(#{args})"
#	end

	def report_mail(subject, variables)
		@main_server.report_mail(subject,variables)
	end
	
	def exit
		@server.stop_service
		super
	end
	
	def serving?
		@state==SERVING
	end

	def to_s
		"[ManqodDb #{name}(#{Process.pid})]"
	end
end

