#this file is part of manqod
#manqod is distributed under the CDDL licence
#the owner of manqod is Dobai-Pataky Balint(dpblnt@gmail.com)

require('mysql')

class MySQL < Mysql
	include Eprint

	MYSQL_FIELD_TYPE={ 
		Mysql::Field::TYPE_BIT			=> "BIT",
		Mysql::Field::TYPE_BLOB			=> "BLOB",
		Mysql::Field::TYPE_CHAR			=> "CHAR",
		Mysql::Field::TYPE_DATE			=> "DATE",
		Mysql::Field::TYPE_DATETIME	=> "DATETIME",
		Mysql::Field::TYPE_DECIMAL	=> "DECIMAL",
		Mysql::Field::TYPE_DOUBLE		=> "DOUBLE",
		Mysql::Field::TYPE_ENUM			=> "ENUM",
		Mysql::Field::TYPE_FLOAT		=> "FLOAT",
		Mysql::Field::TYPE_INT24		=> "INT24",
		Mysql::Field::TYPE_LONG			=> "LONG",
		Mysql::Field::TYPE_LONGLONG	=> "LONGLONG",
		Mysql::Field::TYPE_NEWDECIMAL=> "NEWDECIMAL",
		Mysql::Field::TYPE_NULL			=> "NULL",
		Mysql::Field::TYPE_SET			=> "SET",
		Mysql::Field::TYPE_SHORT		=> "SHORT",
		Mysql::Field::TYPE_STRING		=> "CHAR",
		Mysql::Field::TYPE_TIME			=> "TIME",
		Mysql::Field::TYPE_TIMESTAMP=> "TIMESTAMP",
		Mysql::Field::TYPE_TINY			=> "TINY",
		Mysql::Field::TYPE_VAR_STRING=> "VARCHAR",
		Mysql::Field::TYPE_YEAR			=> "YEAR"
		
	}

	def initialize(host,user,password,dbase)
#	options(Mysql::CLIENT_INTERACTIVE,1)
		@host=host
		@user=user
		@password=password
		@dbase=dbase
		options(Mysql::OPT_CONNECT_TIMEOUT,1)
		options(Mysql::OPT_READ_TIMEOUT,10)
		options(Mysql::OPT_WRITE_TIMEOUT,10)
		einfo("Mysql Client version #{client_info()}")
		begin
			super(@host,@user,@password,@dbase)
		rescue MysqlError => err
			eerror("connecting: #{err.error}")
		end
	end


	def reconnect
		begin
			real_connect(@host,@user,@password,@dbase) 
		rescue MysqlError => err
			ewarn("reconnecting: #{err.error}")
		end
	end

	def query(sql)
		return nil if sql.nil? or sql.length ==0
		begin
			res=super(sql)
		rescue MysqlError => err
			case err.errno
				when 2006, 2013
					einfo("connection dropped: retrying")
					reconnect
					retry
				when 1062
					#duplicate
					res=-1
				else
					eerror("#{err.error} in\n#{sql}")
					raise "#{err.error}\n#{sql}\n"
			end
		end
		return res
	end

	def fetch_hash(res,with_table=false)
		data=Array.new
 		while row=res.fetch_hash(with_table) do
 			data.push(row)
		end unless res.nil? 
		return data
	end

	def fields(sql)
		fields=Array.new
  	if ret=query(sql)
			ret.fetch_fields.sort{|a,b|"#{a.table}.#{a.name}" <=> "#{b.table}.#{b.name}"}.each{|field|
				a={"name"=>"#{field.table}.#{field.name}","type"=>MYSQL_FIELD_TYPE[field.type], "length"=>field.length}
				a["decimals"]=field.decimals if field.is_num?
				fields.push(a)
				a=nil
			}
		end
		fields
	end

	def rows(sql,with_table=false)
		if res=query(sql)
			fetch_hash(res,with_table)
		end
	end

	def to_s
		"mysql(#{@dbase})"
	end
end

