#this file is part of manqod
#manqod is distributed under the CDDL licence
#the author of manqod is Dobai-Pataky Balint(dpblnt@gmail.com)

module Eprint
	TERM_COLOUR={
		Logger::Severity::DEBUG => 33,
		Logger::Severity::INFO => 32,
		Logger::Severity::WARN => 35,
		Logger::Severity::ERROR => 31,
		Logger::Severity::FATAL => 31,
		Logger::Severity::UNKNOWN => 36
	}
	def eprint(subject,level=Logger::Severity::UNKNOWN)
		begin
	 		ManqodLogger.instance.log(level,"\e[1;37m#{self}\e[0m:\e[#{TERM_COLOUR[level]}m#{subject}\e[0m")
	 	rescue => err
	 		print("#{err}\n")
	 		ManqodLogger.instance.set_level("ERROR")
	 		retry
	 	end
	end

	def edebug(subject)
		eprint(subject,Logger::Severity::DEBUG)
	end

	def einfo(subject)
		eprint(subject,Logger::Severity::INFO)
	end

	def ewarn(subject)
		eprint(subject,Logger::Severity::WARN)
	end

	def eerror(subject)
		eprint(subject,Logger::Severity::ERROR)
	end
	def efatal(subject)
		eprint(subject,Logger::Severity::FATAL)
	end

	def ecode(subject)
		eprint(subject,Logger::Severity::UNKNOWN)
	end

	def report_mail(subject, variables)
		return unless ManqodLogger.instance.report_address
		fork{
		begin
		msg="From: manqod@#{Socket.gethostbyname(Socket.gethostname).first}\nTo: #{ManqodLogger.instance.report_address}\nSubject: #{subject}\n"
		variables.each_pair{|k,v| msg="#{msg}\n#{k}: #{v}\n--------------------"}

			Net::SMTP.start('localhost'){ |smtp|
				smtp.send_message(msg, "manqod_server", ManqodLogger.instance.report_address)
				einfo("report sent: #{subject} to {ManqodLogger.instance.report_address}")
			}
		rescue => err
			eerror("can't send error report via SMTP")
		end
		}
	end

	def eexception(ex,*args)
		eerror("error: #{ex}\n#{ex.backtrace.join("\n")}")
		report_mail(ex,{
			'error'=>ex.inspect,
			'backtrace'=>ex.backtrace.join("\n"),
			'class' => self.class,
			'self'=>self.inspect,
			'arguments' => args.inspect
			})
	end

	def eeval(command,context=self)
  	ret=nil
  	begin
			if context and b=context.getBinding then
				ret=eval(command, b)
				else
				ret=eval(command)
			end
		rescue SyntaxError, NameError => err
	  	eexception(err,:command=>command,:context=>context)
  	end
  	ret
	end
	
	def getBinding
		binding
	end

end

